import pandas as pd
import time
import lxml.etree as et

#COLUMNS = ['sku', 'product-id', 'product-id-type', 'description', 'internal-description', 'price', 'price-additional-info', 'quantity', 'min-quantity-alert', 'state', 'available-start-date', 'available-end-date', 'logistic-class', 'favorite-rank', 'discount-start-date', 'discount-end-date', 'discount-price', 'update-delete', 'leadtime-to-ship', 'min-order-quantity', 'max-order-quantity']

# format has to be X.xx-X.xx where X is arbitrary number of chars and x is one char


def read_multipliers(file):
    multipliers = read_csv(file)
    return_dict = []
    for dict1 in multipliers:
        return_dict.append([dict1['multiplier'], float(dict1['from']), float(dict1['to']), float(dict1['vat'])])
    return return_dict
        
def read_classes():
    logistic_classes = {}
    logistic_class_amounts = {}
    shippings = read_csv('download/shipping.csv')
    for dict1 in shippings:
        logistic_classes[dict1['size']] = f"{str(dict1['from']).replace('.0', '.00')}-{str(dict1['to']).replace('.0', '.00')}-{dict1['amount']}"
        logistic_class_amounts[dict1['size']] = float(dict1['amount'])
    return logistic_classes, logistic_class_amounts
    
def decide_logistic_class(logistic_classes, price):
    price = float(price)
    for logistic_class in logistic_classes:
        range = logistic_classes[logistic_class]
        smaller_price = float(range.split('-')[0])
        bigger_price = float(range.split('-')[1])
        if smaller_price <= price < bigger_price:
            return logistic_class
    print('cannot decide logistic class')
    time.sleep(2)
    return None

def determine_price(multipliers, price):
    price = float(price)
    for mp in multipliers:
        smaller_price = mp[1]
        bigger_price = mp[2]
        if smaller_price <= price <= bigger_price:
            return price * float(mp[0]) * mp[3]
        
def determine_price_after_shipping(price, logistic_class, logistic_class_amounts):
    return round(price - logistic_class_amounts[logistic_class], 2)

def read_sku_list():
    aqq = read_csv('download/bigbang_discounted_skus.csv')
    temp = []
    for x in aqq:
        temp.append(int(x['sku']))
    return temp    
    
def read_csv(csv_file, separator = ';', encoding = 'utf-8', form = 'records', cols = None):
    """
    Function to read csv file to list of dictionaries

    Args:
        csv_file (str): file name of csv to read
        separator (str, optional): separator used when reading csv file. Defaults to ';'.
        encoding (str, optional): encoding used when reading csv_file. Defaults to 'utf-8'.
        form (str, optional): form of reading csv file. Defaults to 'records', every record from CSV as dictionary in a list.
        cols (list, optional): which columns read to list of dictionaries

    Returns:
        list: list of dictionaries, where each dictionary is a record from csv
    """    
    return pd.read_csv(csv_file, sep = separator, encoding = encoding, usecols = cols, keep_default_na=False).to_dict(form)

def read_csv_to_df(csv_file, separator = ';', encoding = 'utf-8', cols = None):
    """
    Function to read csv file to list of dictionaries

    Args:
        csv_file (str): file name of csv to read
        separator (str, optional): separator used when reading csv file. Defaults to ';'.
        encoding (str, optional): encoding used when reading csv_file. Defaults to 'utf-8'.
        form (str, optional): form of reading csv file. Defaults to 'records', every record from CSV as dictionary in a list.
        cols (list, optional): which columns read to list of dictionaries

    Returns:
        list: list of dictionaries, where each dictionary is a record from csv
    """
    return pd.read_csv(csv_file, sep = separator, encoding = encoding, usecols = cols, keep_default_na=False, low_memory=False)

def read_xml_to_df(old_xml):
    entries = []
    for event, elem in et.iterparse(old_xml, strip_cdata = False, encoding = 'utf-8'):
        if elem.tag == "o":
            entries.append({'sku_feed': elem.attrib['id'], 'price_feed': elem.attrib['price'], 'stock_feed': elem.attrib['stock']})
    return pd.DataFrame.from_records(entries)
            
def read_config(name):
    config = []
    df = pd.read_csv(name, sep=';')
    config = df.to_dict('records')
    return config

def save_csv(csv_file, list_of_dicts, separator = ';', encoding = 'utf-8', index = False, cols = None, mode = 'w', header = True):
    """
    Function to save list of dictionaries to csv file

    Args:
        csv_file (str): file name of csv to read
        list_of_dicts (list): list of dictionaries to be saved to csv (one dictionary = one record)
        separator (str, optional): separator used when reading csv file. Defaults to ';'.
        encoding (str, optional): encoding used when reading csv_file. Defaults to 'utf-8'.
        cols (list, optional): which columns read to list of dictionaries

    Returns:
        str: resulting csv formatted as string
    """    
    return pd.DataFrame(list_of_dicts).to_csv(csv_file, sep = separator, encoding = encoding, index = index, columns = cols, mode = mode, header = header)
